#!/usr/bin/perl -w

##################################################################
# list-unprotected-vms.pl
#
# Connects to a NetWorker server and provides a list of all
# unprotected virtual machines.
#
# $Id: list-unprotected-vms.pl,v 1.2 2020/03/29 04:31:57 pmdg Exp $
##################################################################

use strict;
use MIME::Base64;
use JSON;
use Data::Dumper;

###################################################################
# Subroutines 
###################################################################


# extract_vm_names($hashref) returns a list of the virtual machine
# names contained in the passed hashref. Names are returned as
# vCenterServer:VMName
sub extract_vm_names {
	my $argCount = @_+0;
	if ($argCount != 1) {
		die "extract_vm_names(\$hashRef) expected 1 argument, got $argCount"
	}
	
	my %data = %{$_[0]};
	
	my @virtualMachines = ();
	if (defined($data{"vms"})) {
		foreach my $vm (@{$data{"vms"}}) {
			my %vMachine = %$vm;
			my $vCenterName = "";
			my $vmName = "";
			if (defined($vMachine{"name"})) {
				$vmName = $vMachine{"name"};
			}
			if (defined($vMachine{"vCenterHostname"})) {
				$vCenterName = $vMachine{"vCenterHostname"};
			}
			
			if ($vCenterName ne "" && $vmName ne "") {
				push(@virtualMachines,"$vCenterName:$vmName");
			}
		}
	}
	
	return @virtualMachines;
}

# in_list($expr,@list) returns true iff $expr is in list.
sub in_list {
	return 0 if (@_+0 < 2);
	
	my $element = $_[0];
	return 0 if (!defined($element));
	shift @_;
	
	my @list = @_;
	return 0 if (!@_ || @_+0 == 0);
	
	my $foundCount = 0;
	my $e = quotemeta($element);
	foreach my $item (@list) {
		my $i = quotemeta($item);
		$foundCount++ if ($e eq $i);
	}
	
	return $foundCount;
}


###################################################################
# Main 
###################################################################
print "\nFind Unprotected Virtual Machines\n\n";
print "Server? ";
my $server = <>;
chomp $server;
print "Username? ";
my $username = <>;
chomp $username;
print "Password? ";
my $password = <>;
chomp $password;

my $base64 = encode_base64("$username:$password");

# First, get a list of all virtual machines
my $vmDetails = "curl -s -k -X GET -H \"Content-Type: application/json\" -H \"Authorization: Basic $base64\" https://$server:9090/nwrestapi/v3/global/vmware/vms";
my $result = "";
my @allVirtualMachines = ();
my @protectedVirtualMachines = ();
if (open(REST,"$vmDetails 2>&1 |")) {
	while (<REST>) {
		my $line = $_;
		chomp $line;
		$result .= $line;
	}
	close(REST);
	my $text = decode_json($result);
	@allVirtualMachines = extract_vm_names($text);
}

# Now, get a list of all protected virtual machines
$vmDetails = "curl -s -k -X GET -H \"Content-Type: application/json\" -H \"Authorization: Basic $base64\" https://$server:9090/nwrestapi/v3/global/vmware/protectedvms";
$result = "";
if (open(REST,"$vmDetails 2>&1 |")) {
	while (<REST>) {
		my $line = $_;
		chomp $line;
		$result .= $line;
	}
	close(REST);
	my $text = decode_json($result);
	@protectedVirtualMachines = extract_vm_names($text);
}

# Now, produce a list of all unprotected virtual machines
my @unprotectedVMs = ();
foreach my $vm (@allVirtualMachines) {
	if(!in_list($vm,@protectedVirtualMachines)) {
		push(@unprotectedVMs,$vm);
	}
}

my $unprotectedCount = @unprotectedVMs+0;
if ($unprotectedCount == 0) {
	die "All virtual machines are protected\n";
}

# else...
if ($unprotectedCount == 1) {
	print "1 Unprotected Virtual Machine:\n";
} else {
	print "$unprotectedCount Unprotected Virtual Machines:\n";
}

foreach my $unprotected (sort {$a cmp $b} @unprotectedVMs) {
	print "\t$unprotected\n";
}